@@ -2,13 +2,13 @@ from typing import Dict, Any, List, Optional
 from datetime import datetime, timedelta
 from pytz import timezone
 from databricks.sdk import WorkspaceClient
+from pyspark.sql import SparkSession
 import sys
 import time
 import json
 import os
 import logging
 import re
-from pyspark.sql import SparkSession
 import pandas as pd
 from .utils import (
     execute_run_databricks,
@@ -27,7 +27,7 @@ class RuleManager:
     ):
         self.logger = logging.getLogger(__name__)
         logging.basicConfig(level=logging.INFO)
-	    
+
         self.spark = spark
         self.dbutils = dbutils
         self.http_path = http_path
@@ -41,20 +41,28 @@ class RuleManager:
             dbutils.notebook.entry_point.getDbutils().notebook().getContext().apiToken().get()
             if dbutils else None
         )
+
+        self.databricks_client = WorkspaceClient()
+
         self.config: Dict[str, Any] = {}
         self.template: Dict[str, Any] = {}
         self.rules: Dict[str, List[Dict[str, Any]]] = {}
         self.execution_id: Optional[int] = None
-        self.unique_timestamp: str = datetime.utcnow().strftime("%Y%m%d%H%M%S%f")
         self.job_start_time: Optional[float] = None
-        self.databricks_client = WorkspaceClient()
-        
+
         if base_dir:
             self.base_dir = base_dir
         elif os.getenv("BRV_BASE_DIR"):
             self.base_dir = os.getenv("BRV_BASE_DIR")
         else:
-            self.base_dir = os.path.dirname(os.path.abspath(__file__))
+            self.base_dir = os.getcwd()
+    
+    def resolve_relative_path(self, raw_path: str) -> str:
+        """
+        Resolves a given relative or absolute file path using the base_dir.
+        """
+        raw_path = raw_path.strip().lstrip("/\\")
+        return os.path.abspath(os.path.join(self.base_dir, raw_path))
 
     def load_config(self, config_path: str):
         with open(config_path, 'r') as f:
@@ -113,26 +121,34 @@ class RuleManager:
                     os.path.join(os.getcwd(), file_path),  # Current directory
                 ]
 
+                
+
                 # Add user-specified templates path if available
                 if hasattr(self, 'base_dir'):
                     possible_paths.append(os.path.join(self.base_dir, 'sql_templates', file_path))
+
                 if hasattr(self, 'sql_templates_path'):
                     possible_paths.insert(0, os.path.join(self.sql_templates_path, file_path))
 
                 for path in possible_paths:
                     if os.path.exists(path):
-                        self.logger.info(f"Reading local file: {path}")
-                        with open(path, 'r', encoding='utf-8') as f:
-                            return f.read()
+                        full_path = path
+                        break
+                else:
+                    raise FileNotFoundError(f"SQL template not found: {file_path}")
 
-                raise FileNotFoundError(f"SQL file not found locally: {file_path}")
+                self.logger.info(f"Reading local file: {full_path}")
+                try:
+                    with open(full_path, 'r', encoding='utf-8') as file:
+                        return file.read()
+                except UnicodeDecodeError:
+                    self.logger.warning(f"UTF-8 decoding failed, retrying with latin-1 for file: {full_path}")
+                    with open(full_path, 'r', encoding='latin-1') as file:
+                        return file.read()
 
             except Exception as e:
                 self.logger.error(f"Error reading file {file_path}: {str(e)}")
                 raise
-        
-
-        
         else:
             # Download file content from Databricks workspace
             try:
@@ -163,6 +179,7 @@ class RuleManager:
                 'result_sql_txt': os.path.join('business_rule_check', 'result.sql'),
                 'update_bookmark_sql_text': os.path.join('common', 'bookmark_update.sql'),
                 'unique_rule_mapping_sql': os.path.join('common', 'unique_rule_mapping.sql'),
+                'TAGS': os.path.join('tags','tag.sql'),
             }
         elif engine_type == 'postgres':
             paths = {
@@ -170,6 +187,7 @@ class RuleManager:
                 'result_sql_txt': os.path.join('business_rule_check', 'result_postgres.sql'),
                 'update_bookmark_sql_text': os.path.join('common', 'bookmark_update_postgres.sql'),
                 'unique_rule_mapping_sql': os.path.join('common', 'unique_rule_mapping_postgres.sql'),
+                'TAGS': os.path.join('tags','tag.sql'),
             }
         else:
             raise ValueError(f"Unsupported ENGINE_TYPE in rule: '{engine_type}'")
@@ -179,7 +197,6 @@ class RuleManager:
 
         return rule
 
-
     def set_data_parity_check_templates(self, rule: Dict[str, Any]) -> Dict[str, Any]:
         """Load SQL templates for data parity check based on engine type.
         
@@ -231,7 +248,6 @@ class RuleManager:
 
         return rule
 
-    
 
     def set_bookmark_value(self, rule: Dict[str, Any]) -> Dict[str, Any]:
         try:
@@ -240,7 +256,7 @@ class RuleManager:
             rule['BOOKMARK_START_DATE'] = rule.get('DEFAULT_BOOKMARK_START_DATE', '1970-01-01')
             self.logger.info(f"Default bookmark start date: {rule['BOOKMARK_START_DATE']}")
 
-            if rule['BOOKMARK_COLUMN'] == 'TEMP_COL_FOR_BOOKMARK' or rule['BOOKMARK_COLUMN_NAME'] == 'TEMP_COL_FOR_BOOKMARK':
+            if rule['BOOKMARK_COLUMN'] == 'TEMP_COL_FOR_BOOKMARK':
                 return self._handle_temp_bookmark(rule)
 
             return self._handle_actual_bookmark(rule)
@@ -306,6 +322,7 @@ class RuleManager:
 
     def process_fail_sql(self, rule: Dict[str, Any]) -> Dict[str, Any]:
         fail_sql_path = rule.get('FAIL_SQL')
+        
         if not fail_sql_path:
             rule['STATUS'] = True
             rule['FAIL_RECORD_COUNT'] = 0
@@ -536,7 +553,7 @@ class RuleManager:
                 else:
                     final_sql = final_sql + ' \n' + line
         return final_sql
-    
+
     def get_query_metadata_col_list(self, sql):
         engine_type = self.template.get('ENGINE_TYPE', '').lower()
         if engine_type == 'databricks':
@@ -759,8 +776,6 @@ class RuleManager:
             #     logging.warning("Attribute Summary DataFrame is empty — skipping CSV write.")
             logging.info("Loaded reports to log tables")
 
-
-
             rule['STATUS'] = True
 
         except Exception as e:
@@ -772,20 +787,50 @@ class RuleManager:
 
 
     def business_rule_check(self) -> Dict[str, List[Dict[str, Any]]]:
+        results = []
         if self.template.get("RULE_CATEGORY_NAME") == 'BUSINESS_RULE_CHECK':
             for idx, rule in enumerate(self.rules.get("RULES", [])):
                 
                 rule = {**self.template, **rule}
                 rule = self.set_system_variables(rule)
+
                 rule = self.set_business_rule_check_templates(rule)
                 rule = self.set_bookmark_value(rule)
                 rule = self.process_fail_sql(rule)
                 rule = self.process_pass_sql(rule)
                 rule = self.set_business_rule_check_final_queries(rule)
                 rule = self.exec_business_rule_check_final_queries(rule)
-                self.rules["RULES"][idx] = rule  
+
+                self.rules["RULES"][idx] = rule
                 self.logger.info(json.dumps(rule, indent=2))
-            
+
+                rule_name = rule.get('RULE_NAME', 'Unknown')
+                rule_id = rule.get('RULE_ID', 'Unknown')
+
+                results.append({
+                "RULE_ID": rule_id,
+                "RULE_NAME": rule_name,
+                "DOMAIN_NAME": rule.get('DOMAIN_NAME'),
+                "TABLES_CHECKED": rule.get('TABLES_CHECKED'),
+                "TEAM_NAME": rule.get('TEAM_NAME'),
+                "SEVERITY": rule.get('SEVERITY'),
+                "RULE_CATEGORY": rule.get('RULE_CATEGORY'),
+                "STATUS": rule.get('STATUS'),
+                "QUERY": rule.get('FAILED_QUERY'),
+                "FAIL_RECORD_COUNT": rule.get('FAIL_RECORD_COUNT'),
+                "PASS_RECORD_COUNT": rule.get('PASS_RECORD_COUNT'),
+                "COMMENTS": rule.get('COMMENTS'),
+                "REMARKS": rule.get('REMARKS'),
+                "TIME": rule.get('TIME', datetime.now(timezone('UTC')).isoformat())
+                })
+
+                if rule.get('STOP_ON_FAIL_STATUS')=='TRUE':  # for checking as per user input
+                    self.logger.warning(f"Rule {rule_name} (ID: {rule_id}) failed with STOP_ON_FAIL_STATUS=TRUE. Stopping execution of remaining rules.")
+                     
+                    if not rule.get('STATUS'): # for checking as per rule status
+                        self.logger.warning(f"Rule {rule_name} (ID: {rule_id}) failed STATUS is FALSE.")         
+                        break
+                  
         elif self.template.get("RULE_CATEGORY_NAME") == 'DATA_PARITY_CHECK':    
             for idx, rule in enumerate(self.rules.get("RULES", [])):
                 rule = {**self.template, **rule}
@@ -802,5 +847,7 @@ class RuleManager:
                     
                 self.logger.info(f"Rule {idx + 1} processed successfully")
 
-         
-        return self.rules
+                results.append(self.rules)
+
+        print(json.dumps({"RULES": results}, indent=2))
+        return results
\ No newline at end of file
